`timescale 1 ns/ 1 ps
module tb();

	// registers
	reg 			clk_ref					;
	reg 			clk_rxmac				;
	reg 			clk_status				;
	reg 			clk_txmac				;
	reg 			pcs_rx_arst_ST			;
	reg 			pcs_tx_arst_ST			;
	reg 			pma_arst_ST				;
	reg [1399:0]	reconfig_to_xcvr		;
	reg [15:0]		status_addr				;
	reg 			status_read				;
	reg 			status_write			;
	reg [31:0]		status_writedata		;
	reg [39:0]		tx_mii_c				;
	reg [319:0]		tx_mii_d				;

	// wires
	wire 			lanes_deskewed			;
	wire [919:0]	reconfig_from_xcvr		;
	wire [39:0]		rx_mii_c				;
	wire [319:0]	rx_mii_d				;
	wire 			rx_mii_valid			;
	wire [31:0]		status_readdata			;
	wire 			status_readdata_valid	;
	wire 			tx_lanes_stable			;
	wire 			tx_mii_ready			;
	wire 			tx_mii_valid			;
	wire [9:0]		tx_serial				;
	wire [9:0]		rx_serial				;

	always begin
		clk_ref <= 1'h1;	#0.775;	clk_ref <= 1'h0;	#0.775;
	end
	
	always begin
		clk_rxmac <= 1'h1;	#1.586;	clk_rxmac <= 1'h0;	#1.586;
	end
	
	always begin
		clk_status <= 1'h1;	#5.000;	clk_status <= 1'h0;	#5.000;
	end
	
	always begin
		clk_txmac <= 1'h1;	#1.586;	clk_txmac <= 1'h0;	#1.586;
	end



e100g dut (
	.clk_ref				( clk_ref				),
	.clk_rxmac				( clk_rxmac				),
	.clk_status				( clk_status			),
	.clk_txmac				( clk_txmac				),
	.lanes_deskewed			( lanes_deskewed		),
	.pcs_rx_arst_ST			( pcs_rx_arst_ST		),
	.pcs_tx_arst_ST			( pcs_tx_arst_ST		),
	.pma_arst_ST			( pma_arst_ST			),
	.reconfig_from_xcvr		( reconfig_from_xcvr	),
	.reconfig_to_xcvr		( reconfig_to_xcvr		),
	.rx_mii_c				( rx_mii_c				),
	.rx_mii_d				( rx_mii_d				),
	.rx_mii_valid			( rx_mii_valid			),
	.rx_serial				( rx_serial				),
	.status_addr			( status_addr			),
	.status_read			( status_read			),
	.status_readdata		( status_readdata		),
	.status_readdata_valid	( status_readdata_valid	),
	.status_write			( status_write			),
	.status_writedata		( status_writedata		),
	.tx_lanes_stable		( tx_lanes_stable		),
	.tx_mii_c				( tx_mii_c				),
	.tx_mii_d				( tx_mii_d				),
	.tx_mii_ready			( tx_mii_ready			),
	.tx_mii_valid			( tx_mii_valid			),
	.tx_serial				( tx_serial				) 
);

	defparam tb.dut.e100g_inst.FAST_SIMULATION = 1;		// Fast simulation mode to reduce the deskew duration



	assign tx_mii_valid = tx_mii_ready;		// Valid should be deasserted to avoid FIFO overflow
	assign rx_serial = tx_serial;



// TASK: CSR READ ------------------------------------------------------------------------------
	task avalon_mm_csr_rd;
		input	[31:0]	address;
	begin
		@(posedge clk_status);
		status_addr 	 = address	;
		status_read 	 = 1'h1		;
		while (!status_readdata_valid) begin
			@(posedge clk_status);
			status_addr 	 = 16'h0000	;
			status_read 	 = 1'h0		;
		end
		wait (status_readdata_valid) begin
			@(posedge clk_status);
			$display("** avalon_mm_csr_rd completed");	
		end
	end
	endtask

// TASK: CSR WRITE ------------------------------------------------------------------------------
	task avalon_mm_csr_wr;
		input [15:0] address;
		input [31:0] writedata;
	begin
	  	@(posedge clk_status);
		status_addr			= address	;
		status_write		= 1'h1		;
		status_writedata	= writedata	;
		@(posedge clk_status);
		status_addr			= 16'h0		;
		status_write		= 1'h0		;
		status_writedata	= 32'h0		;
	end
	endtask



// TASK: SEND_PACKETS ------------------------------------------------------------------------------
	task send_packets;
	input  [31:0]  number_of_packets;
	integer i;
	begin
	fork
		for (i = 1; i <= number_of_packets; i = i +1) begin
			$display("** Sending Packet %d...",i);
			wait_for_ready;  
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0081_cba9_8765_4321_cba9_8765_4321_D555_5555_55FB	;	// Byte 0 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1	;
			wait_for_ready;
			tx_mii_d = 320'h0707_0707_0707_07FD_FFFF_FFFF_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;	// IPG 8 bytes
			tx_mii_c =  40'b1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0081_cba9_8765_4321_cba9_8765_4321_D555_5555_55FB	;	// Byte 0 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1	;
			wait_for_ready;
			tx_mii_d = 320'h0707_0707_0707_07FD_FFFF_FFFF_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;	// IPG 16 bytes
			tx_mii_c =  40'b1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0081_cba9_8765_4321_cba9_8765_4321_D555_5555_55FB_0707_0707_0707_0707	;	// Byte 8 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1	;
			wait_for_ready;
			tx_mii_d = 320'hFFFF_FFFF_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0081_cba9_8765_4321_cba9_8765_4321_D555_5555_55FB_0707_0707_0707_07FD	;	// IPG 8 bytes, 	// Byte 8 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1	;
			wait_for_ready;
			tx_mii_d = 320'hFFFF_FFFF_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h0000_0000_0081_cba9_8765_4321_cba9_8765_4321_D555_5555_55FB_0707_0707_0707_0707_0707_0707_0707_07FD	;	// IPG 16 bytes, 	// Byte 16 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1__1_1__1_1__1_1__1_1	;
			wait_for_ready;
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h0000_0000_0081_cba9_8765_4321_cba9_8765_4321_D555_5555_55FB_0707_0707_0707_07FD_FFFF_FFFF_0000_0000	;	// IPG 8 bytes, 	// Byte 16 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0	;
			wait_for_ready;
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h8765_4321_cba9_8765_4321_D555_5555_55FB_0707_0707_0707_0707_0707_0707_0707_07FD_FFFF_FFFF_0000_0000	;	// IPG 16 bytes, 	// Byte 24 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1__1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0	;
			wait_for_ready;
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0081_cba9	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h8765_4321_cba9_8765_4321_D555_5555_55FB_0707_0707_0707_07FD_FFFF_FFFF_0000_0000_0000_0000_0000_0000	;	// IPG 8 bytes, 	// Byte 24 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;
			wait_for_ready;
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0081_cba9	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h4321_D555_5555_55FB_0707_0707_0707_0707_0707_0707_0707_07FD_FFFF_FFFF_0000_0000_0000_0000_0000_0000	;	// IPG 16 bytes, 	// Byte 32 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1__1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;
			wait_for_ready;
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0081_cba9_8765_4321_cba9_8765	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;

			wait_for_ready;  
			tx_mii_d = 320'h4321_D555_5555_55FB_0707_0707_0707_07FD_FFFF_FFFF_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;	// IPG 8 bytes, 	// Byte 32 Start
			tx_mii_c =  40'b0_0__0_0__0_0__0_1__1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;
			wait_for_ready;
			tx_mii_d = 320'h0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0081_cba9_8765_4321_cba9_8765	;
			tx_mii_c =  40'b0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;
			wait_for_ready;  
			tx_mii_d = 320'h0707_0707_0707_0707_0707_0707_0707_07FD_FFFF_FFFF_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000	;	// IPG 16 bytes
			tx_mii_c =  40'b1_1__1_1__1_1__1_1__1_1__1_1__1_1__1_1__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0__0_0	;
		end
	join
	end
	endtask



	task wait_for_ready;
		#1
		if(!tx_mii_ready) begin
			while(!tx_mii_ready) @(posedge clk_txmac);
		end
		else begin
			@(posedge clk_txmac);
		end
	endtask



// INITIAL -----------------------------------------------------------------------------------------
initial begin
	$display("Running testbench");
	pcs_rx_arst_ST		=    1'h1;
	pcs_tx_arst_ST		=    1'h1;
	pma_arst_ST			=    1'h1;
	reconfig_to_xcvr	= 1400'h0;
	status_addr			=   16'h0;
	status_read			=    1'h0;
	status_write		=    1'h0;
	status_writedata	=   32'h0;
	tx_mii_c			= 40'hFF_FFFF_FFFF;
	tx_mii_d			= {40{8'h07}};

	// Reset deassertion
	#100;
	pcs_rx_arst_ST		=    1'h0;
	pcs_tx_arst_ST		=    1'h0;
	pma_arst_ST			=    1'h0;

	// CSR Scratch read, write, read
	repeat (100) begin @(posedge clk_status); end
	avalon_mm_csr_rd(16'h0001);
	$display("CSR: Scratch read completed", $time);

	avalon_mm_csr_wr(16'h0001, 32'h1234_5678);
	$display("CSR: Scratch write completed", $time);

	avalon_mm_csr_rd(16'h0001);
	$display("CSR: Scratch read completed", $time);

	repeat (20) begin	@(posedge clk_status);	end
	avalon_mm_csr_rd(16'h001D);
	$display("CSR: PCS PMA reset assertion", $time);
	avalon_mm_csr_wr(16'h001D, 32'h0000_0007);
	avalon_mm_csr_rd(16'h001D);
	$display("CSR: PCS PMA reset deassertion", $time);
	avalon_mm_csr_wr(16'h001D, 32'h0000_0000);
	avalon_mm_csr_rd(16'h001D);
	$display("CSR: PCS PMA resets completed", $time);

	// 
	wait (tx_lanes_stable);
	$display("tx_lanes_stable", $time);

	wait (lanes_deskewed);
	$display("lanes_deskewed", $time);

	repeat (100) begin @(posedge clk_txmac); end

	// Sending data
	send_packets(10);
	wait_for_ready;
	
	// Sending IDLE
	tx_mii_c = 40'hFF_FFFF_FFFF;	tx_mii_d = {40{8'h07}};

	repeat (200) begin @(posedge clk_txmac); end
	$stop;
end



endmodule
